@@ -1,7 +1,9 @@
 
 import os, sys, urllib
+import codecs
 from twisted.python import usage
-
+from allmydata.util.encodingutil import unicode_to_url, quote_output
+from allmydata.util.assertutil import precondition
 
 class BaseOptions:
     # unit tests can override these to point at StringIO instances
@@ -17,12 +19,12 @@ class BaseOptions:
 
     def opt_version(self):
         import allmydata
-        print allmydata.get_package_versions_string()
+        print >>self.stdout, allmydata.get_package_versions_string()
         sys.exit(0)
 
     def opt_version_and_path(self):
         import allmydata
-        print allmydata.get_package_versions_string(show_paths=True)
+        print >>self.stdout, allmydata.get_package_versions_string(show_paths=True)
         sys.exit(0)
 
 
@@ -39,7 +41,6 @@ class BasedirMixin:
         self['basedirs'] = [os.path.abspath(os.path.expanduser(b)) for b in self.basedirs]
 
     def parseArgs(self, *args):
-        from allmydata.util.assertutil import precondition
         self.basedirs = []
         if self['basedir']:
             precondition(isinstance(self['basedir'], (str, unicode)), self['basedir'])
@@ -66,7 +67,6 @@ class BasedirMixin:
 
 class NoDefaultBasedirMixin(BasedirMixin):
     def parseArgs(self, *args):
-        from allmydata.util.assertutil import precondition
         # create-client won't default to --basedir=~/.tahoe
         self.basedirs = []
         if self['basedir']:
@@ -84,7 +84,7 @@ class NoDefaultBasedirMixin(BasedirMixin):
         if not self.basedirs:
             raise usage.UsageError("--basedir must be provided")
 
-DEFAULT_ALIAS = "tahoe"
+DEFAULT_ALIAS = u"tahoe"
 
 
 def get_aliases(nodedir):
@@ -96,18 +96,18 @@ def get_aliases(nodedir):
         f = open(rootfile, "r")
         rootcap = f.read().strip()
         if rootcap:
-            aliases["tahoe"] = uri.from_string_dirnode(rootcap).to_string()
+            aliases[u"tahoe"] = uri.from_string_dirnode(rootcap).to_string()
     except EnvironmentError:
         pass
     try:
-        f = open(aliasfile, "r")
+        f = codecs.open(aliasfile, "r", "utf-8")
         for line in f.readlines():
             line = line.strip()
             if line.startswith("#") or not line:
                 continue
-            name, cap = line.split(":", 1)
+            name, cap = line.split(u":", 1)
             # normalize it: remove http: prefix, urldecode
-            cap = cap.strip()
+            cap = cap.strip().encode('utf-8')
             aliases[name] = uri.from_string_dirnode(cap).to_string()
     except EnvironmentError:
         pass
@@ -124,20 +124,36 @@ def platform_uses_lettercolon_drivename():
         return True
     return False
 
-class UnknownAliasError(Exception):
-    pass
 
-def get_alias(aliases, path, default):
+class TahoeError(Exception):
+    def __init__(self, msg):
+        Exception.__init__(self, msg)
+        self.msg = msg
+
+    def display(self, err):
+        print >>err, self.msg
+
+
+class UnknownAliasError(TahoeError):
+    def __init__(self, msg):
+        TahoeError.__init__(self, "error: " + msg)
+
+
+def get_alias(aliases, path_unicode, default):
+    """
+    Transform u"work:path/filename" into (aliases[u"work"], u"path/filename".encode('utf-8')).
+    If default=None, then an empty alias is indicated by returning
+    DefaultAliasMarker. We special-case strings with a recognized cap URI
+    prefix, to make it easy to access specific files/directories by their
+    caps.
+    If the transformed alias is either not found in aliases, or is blank
+    and default is not found in aliases, an UnknownAliasError is
+    raised.
+    """
+    precondition(isinstance(path_unicode, unicode), path_unicode)
+
     from allmydata import uri
-    # transform "work:path/filename" into (aliases["work"], "path/filename").
-    # If default=None, then an empty alias is indicated by returning
-    # DefaultAliasMarker. We special-case strings with a recognized cap URI
-    # prefix, to make it easy to access specific files/directories by their
-    # caps.
-    # If the transformed alias is either not found in aliases, or is blank
-    # and default is not found in aliases, an UnknownAliasError is
-    # raised.
-    path = path.strip()
+    path = path_unicode.encode('utf-8').strip(" ")
     if uri.has_uri_prefix(path):
         # We used to require "URI:blah:./foo" in order to get a subpath,
         # stripping out the ":./" sequence. We still allow that for compatibility,
@@ -159,12 +175,14 @@ def get_alias(aliases, path, default):
                                     "'tahoe' alias doesn't exist. To create "
                                     "it, use 'tahoe create-alias tahoe'.")
         return aliases[default], path
-    if colon == 1 and default == None and platform_uses_lettercolon_drivename():
+    if colon == 1 and default is None and platform_uses_lettercolon_drivename():
         # treat C:\why\must\windows\be\so\weird as a local path, not a tahoe
         # file in the "C:" alias
         return DefaultAliasMarker, path
-    alias = path[:colon]
-    if "/" in alias:
+
+    # decoding must succeed because path is valid UTF-8 and colon & space are ASCII
+    alias = path[:colon].decode('utf-8')
+    if u"/" in alias:
         # no alias, but there's a colon in a dirname/filename, like
         # "foo/bar:7"
         if default == None:
@@ -175,9 +193,10 @@ def get_alias(aliases, path, default):
                                     "it, use 'tahoe create-alias tahoe'.")
         return aliases[default], path
     if alias not in aliases:
-        raise UnknownAliasError("Unknown alias '%s', please create it with 'tahoe add-alias' or 'tahoe create-alias'." % alias)
+        raise UnknownAliasError("Unknown alias %s, please create it with 'tahoe add-alias' or 'tahoe create-alias'." %
+                                quote_output(alias))
     return aliases[alias], path[colon+1:]
 
 def escape_path(path):
     segments = path.split("/")
-    return "/".join([urllib.quote(s) for s in segments])
+    return "/".join([urllib.quote(unicode_to_url(s)) for s in segments])
