@@ -4,7 +4,6 @@
 import struct, time, os
 from twisted.python import usage, failure
 from twisted.internet import defer
-from allmydata.scripts.cli import VDriveOptions
 
 class DumpOptions(usage.Options):
     def getSynopsis(self):
@@ -758,22 +757,6 @@ def repl(options):
     return code.interact()
 
 
-class ConsolidateOptions(VDriveOptions):
-    optParameters = [
-        ("dbfile", None, None, "persistent file for reusable dirhashes"),
-        ("backupfile", "b", None, "file to store backup of Archives/ contents"),
-        ]
-    optFlags = [
-        ("really", None, "Really remove old snapshot directories"),
-        ("verbose", "v", "Emit a line for every directory examined"),
-        ]
-    def parseArgs(self, where):
-        self.where = where
-
-def consolidate(options):
-    from consolidate import main; return main(options)
-
-
 class DebugCommand(usage.Options):
     subCommands = [
         ["dump-share", None, DumpOptions,
@@ -783,7 +766,6 @@ class DebugCommand(usage.Options):
         ["catalog-shares", None, CatalogSharesOptions, "Describe shares in node dirs"],
         ["corrupt-share", None, CorruptShareOptions, "Corrupt a share"],
         ["repl", None, ReplOptions, "Open a python interpreter"],
-        ["consolidate", None, ConsolidateOptions, "Consolidate non-shared backups"],
         ]
     def postOptions(self):
         if not hasattr(self, 'subOptions'):
@@ -799,7 +781,6 @@ Subcommands:
     tahoe debug find-shares     Locate sharefiles in node directories
     tahoe debug catalog-shares  Describe all shares in node dirs
     tahoe debug corrupt-share   Corrupt a share by flipping a bit.
-    tahoe debug consolidate     Consolidate old non-shared backups into shared ones.
 
 Please run e.g. 'tahoe debug dump-share --help' for more details on each
 subcommand.
@@ -813,7 +794,6 @@ subDispatch = {
     "catalog-shares": catalog_shares,
     "corrupt-share": corrupt_share,
     "repl": repl,
-    "consolidate": consolidate,
     }
 
 
