@@ -21,7 +21,7 @@ from allmydata.util.assertutil import precondition
 from allmydata.util.consumer import download_to_data
 from allmydata.stats import StatsGathererService
 from allmydata.key_generator import KeyGeneratorService
-import common_util as testutil
+import allmydata.test.common_util as testutil
 from allmydata import immutable
 
 
@@ -79,16 +79,16 @@ class FakeCHKFileNode:
         data["count-recoverable-versions"] = 1
         data["count-unrecoverable-versions"] = 0
         if is_bad:
-             r.set_healthy(False)
-             r.set_recoverable(True)
-             data["count-shares-good"] = 9
-             data["list-corrupt-shares"] = [(nodeid, self.storage_index, 0)]
-             r.problems = failure.Failure(CorruptShareError(is_bad))
+            r.set_healthy(False)
+            r.set_recoverable(True)
+            data["count-shares-good"] = 9
+            data["list-corrupt-shares"] = [(nodeid, self.storage_index, 0)]
+            r.problems = failure.Failure(CorruptShareError(is_bad))
         else:
-             r.set_healthy(True)
-             r.set_recoverable(True)
-             data["count-shares-good"] = 10
-             r.problems = []
+            r.set_healthy(True)
+            r.set_recoverable(True)
+            data["count-shares-good"] = 10
+            r.problems = []
         r.set_data(data)
         r.set_needs_rebalancing(False)
         return defer.succeed(r)
@@ -250,17 +250,17 @@ class FakeMutableFileNode:
         data["count-recoverable-versions"] = 1
         data["count-unrecoverable-versions"] = 0
         if is_bad:
-             r.set_healthy(False)
-             r.set_recoverable(True)
-             data["count-shares-good"] = 9
-             r.problems = failure.Failure(CorruptShareError("peerid",
-                                                            0, # shnum
-                                                            is_bad))
+            r.set_healthy(False)
+            r.set_recoverable(True)
+            data["count-shares-good"] = 9
+            r.problems = failure.Failure(CorruptShareError("peerid",
+                                                           0, # shnum
+                                                           is_bad))
         else:
-             r.set_healthy(True)
-             r.set_recoverable(True)
-             data["count-shares-good"] = 10
-             r.problems = []
+            r.set_healthy(True)
+            r.set_recoverable(True)
+            data["count-shares-good"] = 10
+            r.problems = []
         r.set_data(data)
         r.set_needs_rebalancing(False)
         return defer.succeed(r)
@@ -938,6 +938,9 @@ class ShareManglingMixin(SystemTestMixin):
             # We need multiple segments to test crypttext hash trees that are
             # non-trivial (i.e. they have more than just one hash in them).
             cl0.DEFAULT_ENCODING_PARAMETERS['max_segment_size'] = 12
+            # Tests that need to test servers of happiness using this should
+            # set their own value for happy -- the default (7) breaks stuff.
+            cl0.DEFAULT_ENCODING_PARAMETERS['happy'] = 1
             d2 = cl0.upload(immutable.upload.Data(TEST_DATA, convergence=""))
             def _after_upload(u):
                 filecap = u.uri
@@ -953,7 +956,7 @@ class ShareManglingMixin(SystemTestMixin):
         d.addCallback(_stash_it)
         return d
 
-    def find_shares(self, unused=None):
+    def find_all_shares(self, unused=None):
         """Locate shares on disk. Returns a dict that maps
         (clientnum,sharenum) to a string that contains the share container
         (copied directly from the disk, containing leases etc). You can
@@ -978,7 +981,7 @@ class ShareManglingMixin(SystemTestMixin):
 
     def replace_shares(self, newshares, storage_index):
         """Replace shares on disk. Takes a dictionary in the same form
-        as find_shares() returns."""
+        as find_all_shares() returns."""
 
         for i, c in enumerate(self.clients):
             sharedir = c.getServiceNamed("storage").sharedir
@@ -1003,7 +1006,7 @@ class ShareManglingMixin(SystemTestMixin):
     def _delete_a_share(self, unused=None, sharenum=None):
         """ Delete one share. """
 
-        shares = self.find_shares()
+        shares = self.find_all_shares()
         ks = shares.keys()
         if sharenum is not None:
             k = [ key for key in shares.keys() if key[1] == sharenum ][0]
@@ -1015,7 +1018,7 @@ class ShareManglingMixin(SystemTestMixin):
         return unused
 
     def _corrupt_a_share(self, unused, corruptor_func, sharenum):
-        shares = self.find_shares()
+        shares = self.find_all_shares()
         ks = [ key for key in shares.keys() if key[1] == sharenum ]
         assert ks, (shares.keys(), sharenum)
         k = ks[0]
@@ -1025,14 +1028,14 @@ class ShareManglingMixin(SystemTestMixin):
 
     def _corrupt_all_shares(self, unused, corruptor_func):
         """ All shares on disk will be corrupted by corruptor_func. """
-        shares = self.find_shares()
+        shares = self.find_all_shares()
         for k in shares.keys():
             self._corrupt_a_share(unused, corruptor_func, k[1])
         return corruptor_func
 
     def _corrupt_a_random_share(self, unused, corruptor_func):
         """ Exactly one share on disk will be corrupted by corruptor_func. """
-        shares = self.find_shares()
+        shares = self.find_all_shares()
         ks = shares.keys()
         k = random.choice(ks)
         self._corrupt_a_share(unused, corruptor_func, k[1])
@@ -1087,8 +1090,8 @@ class ShouldFailMixin:
         error message, if any, because Deferred chains frequently make it
         difficult to tell which assertion was tripped.
 
-        The substring= argument, if not None, must appear inside the
-        stringified Failure, or the test will fail.
+        The substring= argument, if not None, must appear in the 'repr'
+        of the message wrapped by this Failure, or the test will fail.
         """
 
         assert substring is None or isinstance(substring, str)
@@ -1097,9 +1100,10 @@ class ShouldFailMixin:
             if isinstance(res, failure.Failure):
                 res.trap(expected_failure)
                 if substring:
-                    self.failUnless(substring in str(res),
+                    message = repr(res.value.args[0])
+                    self.failUnless(substring in message,
                                     "substring '%s' not in '%s'"
-                                    % (substring, str(res)))
+                                    % (substring, message))
             else:
                 self.fail("%s was supposed to raise %s, not get '%s'" %
                           (which, expected_failure, res))
