@@ -4,15 +4,16 @@ from allmydata import uri
 from allmydata.util import hashutil, base32
 from allmydata.interfaces import IURI, IFileURI, IDirnodeURI, IMutableFileURI, \
     IVerifierURI, CapConstraintError
+import allmydata.test.common_util as testutil
 
-class Literal(unittest.TestCase):
+class Literal(testutil.ReallyEqualMixin, unittest.TestCase):
     def _help_test(self, data):
         u = uri.LiteralFileURI(data)
         self.failUnless(IURI.providedBy(u))
         self.failUnless(IFileURI.providedBy(u))
         self.failIf(IDirnodeURI.providedBy(u))
-        self.failUnlessEqual(u.data, data)
-        self.failUnlessEqual(u.get_size(), len(data))
+        self.failUnlessReallyEqual(u.data, data)
+        self.failUnlessReallyEqual(u.get_size(), len(data))
         self.failUnless(u.is_readonly())
         self.failIf(u.is_mutable())
 
@@ -20,26 +21,26 @@ class Literal(unittest.TestCase):
         self.failUnless(IURI.providedBy(u2))
         self.failUnless(IFileURI.providedBy(u2))
         self.failIf(IDirnodeURI.providedBy(u2))
-        self.failUnlessEqual(u2.data, data)
-        self.failUnlessEqual(u2.get_size(), len(data))
+        self.failUnlessReallyEqual(u2.data, data)
+        self.failUnlessReallyEqual(u2.get_size(), len(data))
         self.failUnless(u2.is_readonly())
         self.failIf(u2.is_mutable())
 
         u2i = uri.from_string(u.to_string(), deep_immutable=True)
         self.failUnless(IFileURI.providedBy(u2i))
         self.failIf(IDirnodeURI.providedBy(u2i))
-        self.failUnlessEqual(u2i.data, data)
-        self.failUnlessEqual(u2i.get_size(), len(data))
+        self.failUnlessReallyEqual(u2i.data, data)
+        self.failUnlessReallyEqual(u2i.get_size(), len(data))
         self.failUnless(u2i.is_readonly())
         self.failIf(u2i.is_mutable())
 
         u3 = u.get_readonly()
         self.failUnlessIdentical(u, u3)
-        self.failUnlessEqual(u.get_verify_cap(), None)
+        self.failUnlessReallyEqual(u.get_verify_cap(), None)
 
         he = u.to_human_encoding()
         u_h = uri.LiteralFileURI.init_from_human_encoding(he)
-        self.failUnlessEqual(u, u_h)
+        self.failUnlessReallyEqual(u, u_h)
 
     def test_empty(self):
         data = "" # This data is some *very* small data!
@@ -53,7 +54,7 @@ class Literal(unittest.TestCase):
         data = "This contains \x00 and URI:LIT: and \n, oh my."
         return self._help_test(data)
 
-class Compare(unittest.TestCase):
+class Compare(testutil.ReallyEqualMixin, unittest.TestCase):
     def test_compare(self):
         lit1 = uri.LiteralFileURI("some data")
         fileURI = 'URI:CHK:f5ahxa25t4qkktywz6teyfvcx4:opuioq7tj2y6idzfp6cazehtmgs5fdcebcz3cygrxyydvcozrmeq:3:10:345834'
@@ -61,11 +62,11 @@ class Compare(unittest.TestCase):
         chk2 = uri.CHKFileURI.init_from_string(fileURI)
         unk = uri.UnknownURI("lafs://from_the_future")
         self.failIfEqual(lit1, chk1)
-        self.failUnlessEqual(chk1, chk2)
+        self.failUnlessReallyEqual(chk1, chk2)
         self.failIfEqual(chk1, "not actually a URI")
         # these should be hashable too
         s = set([lit1, chk1, chk2, unk])
-        self.failUnlessEqual(len(s), 3) # since chk1==chk2
+        self.failUnlessReallyEqual(len(s), 3) # since chk1==chk2
 
     def test_is_uri(self):
         lit1 = uri.LiteralFileURI("some data").to_string()
@@ -89,7 +90,7 @@ class Compare(unittest.TestCase):
         self.failIf(uri.has_uri_prefix(None))
         self.failIf(uri.has_uri_prefix("foo"))
 
-class CHKFile(unittest.TestCase):
+class CHKFile(testutil.ReallyEqualMixin, unittest.TestCase):
     def test_pack(self):
         key = "\x00\x01\x02\x03\x04\x05\x06\x07\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f"
         storage_index = hashutil.storage_index_hash(key)
@@ -102,45 +103,45 @@ class CHKFile(unittest.TestCase):
                            needed_shares=needed_shares,
                            total_shares=total_shares,
                            size=size)
-        self.failUnlessEqual(u.get_storage_index(), storage_index)
-        self.failUnlessEqual(u.key, key)
-        self.failUnlessEqual(u.uri_extension_hash, uri_extension_hash)
-        self.failUnlessEqual(u.needed_shares, needed_shares)
-        self.failUnlessEqual(u.total_shares, total_shares)
-        self.failUnlessEqual(u.size, size)
+        self.failUnlessReallyEqual(u.get_storage_index(), storage_index)
+        self.failUnlessReallyEqual(u.key, key)
+        self.failUnlessReallyEqual(u.uri_extension_hash, uri_extension_hash)
+        self.failUnlessReallyEqual(u.needed_shares, needed_shares)
+        self.failUnlessReallyEqual(u.total_shares, total_shares)
+        self.failUnlessReallyEqual(u.size, size)
         self.failUnless(u.is_readonly())
         self.failIf(u.is_mutable())
         self.failUnless(IURI.providedBy(u))
         self.failUnless(IFileURI.providedBy(u))
         self.failIf(IDirnodeURI.providedBy(u))
-        self.failUnlessEqual(u.get_size(), 1234)
+        self.failUnlessReallyEqual(u.get_size(), 1234)
 
         u_ro = u.get_readonly()
         self.failUnlessIdentical(u, u_ro)
         he = u.to_human_encoding()
-        self.failUnlessEqual(he, "http://127.0.0.1:3456/uri/" + u.to_string())
-        self.failUnlessEqual(uri.CHKFileURI.init_from_human_encoding(he), u)
+        self.failUnlessReallyEqual(he, "http://127.0.0.1:3456/uri/" + u.to_string())
+        self.failUnlessReallyEqual(uri.CHKFileURI.init_from_human_encoding(he), u)
 
         u2 = uri.from_string(u.to_string())
-        self.failUnlessEqual(u2.get_storage_index(), storage_index)
-        self.failUnlessEqual(u2.key, key)
-        self.failUnlessEqual(u2.uri_extension_hash, uri_extension_hash)
-        self.failUnlessEqual(u2.needed_shares, needed_shares)
-        self.failUnlessEqual(u2.total_shares, total_shares)
-        self.failUnlessEqual(u2.size, size)
+        self.failUnlessReallyEqual(u2.get_storage_index(), storage_index)
+        self.failUnlessReallyEqual(u2.key, key)
+        self.failUnlessReallyEqual(u2.uri_extension_hash, uri_extension_hash)
+        self.failUnlessReallyEqual(u2.needed_shares, needed_shares)
+        self.failUnlessReallyEqual(u2.total_shares, total_shares)
+        self.failUnlessReallyEqual(u2.size, size)
         self.failUnless(u2.is_readonly())
         self.failIf(u2.is_mutable())
         self.failUnless(IURI.providedBy(u2))
         self.failUnless(IFileURI.providedBy(u2))
         self.failIf(IDirnodeURI.providedBy(u2))
-        self.failUnlessEqual(u2.get_size(), 1234)
+        self.failUnlessReallyEqual(u2.get_size(), 1234)
 
         u2i = uri.from_string(u.to_string(), deep_immutable=True)
-        self.failUnlessEqual(u.to_string(), u2i.to_string())
+        self.failUnlessReallyEqual(u.to_string(), u2i.to_string())
         u2ro = uri.from_string(uri.ALLEGED_READONLY_PREFIX + u.to_string())
-        self.failUnlessEqual(u.to_string(), u2ro.to_string())
+        self.failUnlessReallyEqual(u.to_string(), u2ro.to_string())
         u2imm = uri.from_string(uri.ALLEGED_IMMUTABLE_PREFIX + u.to_string())
-        self.failUnlessEqual(u.to_string(), u2imm.to_string())
+        self.failUnlessReallyEqual(u.to_string(), u2imm.to_string())
 
         v = u.get_verify_cap()
         self.failUnless(isinstance(v.to_string(), str))
@@ -148,10 +149,10 @@ class CHKFile(unittest.TestCase):
         self.failIf(v.is_mutable())
 
         v2 = uri.from_string(v.to_string())
-        self.failUnlessEqual(v, v2)
+        self.failUnlessReallyEqual(v, v2)
         he = v.to_human_encoding()
         v2_h = uri.CHKFileVerifierURI.init_from_human_encoding(he)
-        self.failUnlessEqual(v2, v2_h)
+        self.failUnlessReallyEqual(v2, v2_h)
 
         v3 = uri.CHKFileVerifierURI(storage_index="\x00"*16,
                                     uri_extension_hash="\x00"*32,
@@ -192,7 +193,7 @@ class CHKFile(unittest.TestCase):
                               )
 
 
-class Extension(unittest.TestCase):
+class Extension(testutil.ReallyEqualMixin, unittest.TestCase):
     def test_pack(self):
         data = {"stuff": "value",
                 "size": 12,
@@ -201,53 +202,66 @@ class Extension(unittest.TestCase):
                 }
         ext = uri.pack_extension(data)
         d = uri.unpack_extension(ext)
-        self.failUnlessEqual(d["stuff"], "value")
-        self.failUnlessEqual(d["size"], 12)
-        self.failUnlessEqual(d["big_hash"], hashutil.tagged_hash("foo", "bar"))
+        self.failUnlessReallyEqual(d["stuff"], "value")
+        self.failUnlessReallyEqual(d["size"], 12)
+        self.failUnlessReallyEqual(d["big_hash"], hashutil.tagged_hash("foo", "bar"))
 
         readable = uri.unpack_extension_readable(ext)
-        self.failUnlessEqual(readable["needed_shares"], 3)
-        self.failUnlessEqual(readable["stuff"], "value")
-        self.failUnlessEqual(readable["size"], 12)
-        self.failUnlessEqual(readable["big_hash"],
+        self.failUnlessReallyEqual(readable["needed_shares"], 3)
+        self.failUnlessReallyEqual(readable["stuff"], "value")
+        self.failUnlessReallyEqual(readable["size"], 12)
+        self.failUnlessReallyEqual(readable["big_hash"],
                              base32.b2a(hashutil.tagged_hash("foo", "bar")))
-        self.failUnlessEqual(readable["UEB_hash"],
+        self.failUnlessReallyEqual(readable["UEB_hash"],
                              base32.b2a(hashutil.uri_extension_hash(ext)))
 
-class Unknown(unittest.TestCase):
+class Unknown(testutil.ReallyEqualMixin, unittest.TestCase):
     def test_from_future(self):
         # any URI type that we don't recognize should be treated as unknown
         future_uri = "I am a URI from the future. Whatever you do, don't "
         u = uri.from_string(future_uri)
         self.failUnless(isinstance(u, uri.UnknownURI))
-        self.failUnlessEqual(u.to_string(), future_uri)
+        self.failUnlessReallyEqual(u.to_string(), future_uri)
         self.failUnless(u.get_readonly() is None)
         self.failUnless(u.get_error() is None)
 
         u2 = uri.UnknownURI(future_uri, error=CapConstraintError("..."))
-        self.failUnlessEqual(u.to_string(), future_uri)
+        self.failUnlessReallyEqual(u.to_string(), future_uri)
         self.failUnless(u2.get_readonly() is None)
         self.failUnless(isinstance(u2.get_error(), CapConstraintError))
 
-class Constraint(unittest.TestCase):
-    def test_constraint(self):
-       good="http://127.0.0.1:3456/uri/URI%3ADIR2%3Agh3l5rbvnv2333mrfvalmjfr4i%3Alz6l7u3z3b7g37s4zkdmfpx5ly4ib4m6thrpbusi6ys62qtc6mma/"
-       uri.DirectoryURI.init_from_human_encoding(good)
-       self.failUnlessRaises(uri.BadURIError, uri.DirectoryURI.init_from_string, good)
-       bad = good + '==='
-       self.failUnlessRaises(uri.BadURIError, uri.DirectoryURI.init_from_human_encoding, bad)
-       self.failUnlessRaises(uri.BadURIError, uri.DirectoryURI.init_from_string, bad)
-       fileURI = 'URI:CHK:gh3l5rbvnv2333mrfvalmjfr4i:lz6l7u3z3b7g37s4zkdmfpx5ly4ib4m6thrpbusi6ys62qtc6mma:3:10:345834'
-       uri.CHKFileURI.init_from_string(fileURI)
+        # Future caps might have non-ASCII chars in them. (Or maybe not, who can tell about the future?)
+        future_uri = u"I am a cap from the \u263A future. Whatever you ".encode('utf-8')
+        u = uri.from_string(future_uri)
+        self.failUnless(isinstance(u, uri.UnknownURI))
+        self.failUnlessReallyEqual(u.to_string(), future_uri)
+        self.failUnless(u.get_readonly() is None)
+        self.failUnless(u.get_error() is None)
 
-class Mutable(unittest.TestCase):
+        u2 = uri.UnknownURI(future_uri, error=CapConstraintError("..."))
+        self.failUnlessReallyEqual(u.to_string(), future_uri)
+        self.failUnless(u2.get_readonly() is None)
+        self.failUnless(isinstance(u2.get_error(), CapConstraintError))
+
+class Constraint(testutil.ReallyEqualMixin, unittest.TestCase):
+    def test_constraint(self):
+        good="http://127.0.0.1:3456/uri/URI%3ADIR2%3Agh3l5rbvnv2333mrfvalmjfr4i%3Alz6l7u3z3b7g37s4zkdmfpx5ly4ib4m6thrpbusi6ys62qtc6mma/"
+        uri.DirectoryURI.init_from_human_encoding(good)
+        self.failUnlessRaises(uri.BadURIError, uri.DirectoryURI.init_from_string, good)
+        bad = good + '==='
+        self.failUnlessRaises(uri.BadURIError, uri.DirectoryURI.init_from_human_encoding, bad)
+        self.failUnlessRaises(uri.BadURIError, uri.DirectoryURI.init_from_string, bad)
+        fileURI = 'URI:CHK:gh3l5rbvnv2333mrfvalmjfr4i:lz6l7u3z3b7g37s4zkdmfpx5ly4ib4m6thrpbusi6ys62qtc6mma:3:10:345834'
+        uri.CHKFileURI.init_from_string(fileURI)
+
+class Mutable(testutil.ReallyEqualMixin, unittest.TestCase):
     def test_pack(self):
         writekey = "\x01" * 16
         fingerprint = "\x02" * 32
 
         u = uri.WriteableSSKFileURI(writekey, fingerprint)
-        self.failUnlessEqual(u.writekey, writekey)
-        self.failUnlessEqual(u.fingerprint, fingerprint)
+        self.failUnlessReallyEqual(u.writekey, writekey)
+        self.failUnlessReallyEqual(u.fingerprint, fingerprint)
         self.failIf(u.is_readonly())
         self.failUnless(u.is_mutable())
         self.failUnless(IURI.providedBy(u))
@@ -257,11 +271,11 @@ class Mutable(unittest.TestCase):
 
         he = u.to_human_encoding()
         u_h = uri.WriteableSSKFileURI.init_from_human_encoding(he)
-        self.failUnlessEqual(u, u_h)
+        self.failUnlessReallyEqual(u, u_h)
 
         u2 = uri.from_string(u.to_string())
-        self.failUnlessEqual(u2.writekey, writekey)
-        self.failUnlessEqual(u2.fingerprint, fingerprint)
+        self.failUnlessReallyEqual(u2.writekey, writekey)
+        self.failUnlessReallyEqual(u2.fingerprint, fingerprint)
         self.failIf(u2.is_readonly())
         self.failUnless(u2.is_mutable())
         self.failUnless(IURI.providedBy(u2))
@@ -277,8 +291,8 @@ class Mutable(unittest.TestCase):
 
         u3 = u2.get_readonly()
         readkey = hashutil.ssk_readkey_hash(writekey)
-        self.failUnlessEqual(u3.fingerprint, fingerprint)
-        self.failUnlessEqual(u3.readkey, readkey)
+        self.failUnlessReallyEqual(u3.fingerprint, fingerprint)
+        self.failUnlessReallyEqual(u3.readkey, readkey)
         self.failUnless(u3.is_readonly())
         self.failUnless(u3.is_mutable())
         self.failUnless(IURI.providedBy(u3))
@@ -288,17 +302,17 @@ class Mutable(unittest.TestCase):
         u3i = uri.from_string(u3.to_string(), deep_immutable=True)
         self.failUnless(isinstance(u3i, uri.UnknownURI), u3i)
         u3ro = uri.from_string(uri.ALLEGED_READONLY_PREFIX + u3.to_string())
-        self.failUnlessEqual(u3.to_string(), u3ro.to_string())
+        self.failUnlessReallyEqual(u3.to_string(), u3ro.to_string())
         u3imm = uri.from_string(uri.ALLEGED_IMMUTABLE_PREFIX + u3.to_string())
         self.failUnless(isinstance(u3imm, uri.UnknownURI), u3imm)
 
         he = u3.to_human_encoding()
         u3_h = uri.ReadonlySSKFileURI.init_from_human_encoding(he)
-        self.failUnlessEqual(u3, u3_h)
+        self.failUnlessReallyEqual(u3, u3_h)
 
         u4 = uri.ReadonlySSKFileURI(readkey, fingerprint)
-        self.failUnlessEqual(u4.fingerprint, fingerprint)
-        self.failUnlessEqual(u4.readkey, readkey)
+        self.failUnlessReallyEqual(u4.fingerprint, fingerprint)
+        self.failUnlessReallyEqual(u4.readkey, readkey)
         self.failUnless(u4.is_readonly())
         self.failUnless(u4.is_mutable())
         self.failUnless(IURI.providedBy(u4))
@@ -308,28 +322,28 @@ class Mutable(unittest.TestCase):
         u4i = uri.from_string(u4.to_string(), deep_immutable=True)
         self.failUnless(isinstance(u4i, uri.UnknownURI), u4i)
         u4ro = uri.from_string(uri.ALLEGED_READONLY_PREFIX + u4.to_string())
-        self.failUnlessEqual(u4.to_string(), u4ro.to_string())
+        self.failUnlessReallyEqual(u4.to_string(), u4ro.to_string())
         u4imm = uri.from_string(uri.ALLEGED_IMMUTABLE_PREFIX + u4.to_string())
         self.failUnless(isinstance(u4imm, uri.UnknownURI), u4imm)
 
         u4a = uri.from_string(u4.to_string())
-        self.failUnlessEqual(u4a, u4)
+        self.failUnlessReallyEqual(u4a, u4)
         self.failUnless("ReadonlySSKFileURI" in str(u4a))
         self.failUnlessIdentical(u4a.get_readonly(), u4a)
 
         u5 = u4.get_verify_cap()
         self.failUnless(IVerifierURI.providedBy(u5))
-        self.failUnlessEqual(u5.get_storage_index(), u.get_storage_index())
+        self.failUnlessReallyEqual(u5.get_storage_index(), u.get_storage_index())
         u7 = u.get_verify_cap()
         self.failUnless(IVerifierURI.providedBy(u7))
-        self.failUnlessEqual(u7.get_storage_index(), u.get_storage_index())
+        self.failUnlessReallyEqual(u7.get_storage_index(), u.get_storage_index())
 
         he = u5.to_human_encoding()
         u5_h = uri.SSKVerifierURI.init_from_human_encoding(he)
-        self.failUnlessEqual(u5, u5_h)
+        self.failUnlessReallyEqual(u5, u5_h)
 
 
-class Dirnode(unittest.TestCase):
+class Dirnode(testutil.ReallyEqualMixin, unittest.TestCase):
     def test_pack(self):
         writekey = "\x01" * 16
         fingerprint = "\x02" * 32
@@ -347,7 +361,7 @@ class Dirnode(unittest.TestCase):
         self.failIf(u1_filenode.is_readonly())
 
         u2 = uri.from_string(u1.to_string())
-        self.failUnlessEqual(u1.to_string(), u2.to_string())
+        self.failUnlessReallyEqual(u1.to_string(), u2.to_string())
         self.failIf(u2.is_readonly())
         self.failUnless(u2.is_mutable())
         self.failUnless(IURI.providedBy(u2))
@@ -378,7 +392,7 @@ class Dirnode(unittest.TestCase):
         self.failUnlessIdentical(u3a, u3a.get_readonly())
 
         u4 = uri.ReadonlyDirectoryURI(u2._filenode_uri.get_readonly())
-        self.failUnlessEqual(u4.to_string(), u3.to_string())
+        self.failUnlessReallyEqual(u4.to_string(), u3.to_string())
         self.failUnless(u4.is_readonly())
         self.failUnless(u4.is_mutable())
         self.failUnless(IURI.providedBy(u4))
@@ -395,7 +409,7 @@ class Dirnode(unittest.TestCase):
                      ]
         for v in verifiers:
             self.failUnless(IVerifierURI.providedBy(v))
-            self.failUnlessEqual(v._filenode_uri,
+            self.failUnlessReallyEqual(v._filenode_uri,
                                  u1.get_verify_cap()._filenode_uri)
 
     def test_immutable(self):
@@ -411,7 +425,7 @@ class Dirnode(unittest.TestCase):
                                total_shares=total_shares,
                                size=size)
         fncap = fnuri.to_string()
-        self.failUnlessEqual(fncap, "URI:CHK:aeaqcaibaeaqcaibaeaqcaibae:nf3nimquen7aeqm36ekgxomalstenpkvsdmf6fplj7swdatbv5oa:3:10:1234")
+        self.failUnlessReallyEqual(fncap, "URI:CHK:aeaqcaibaeaqcaibaeaqcaibae:nf3nimquen7aeqm36ekgxomalstenpkvsdmf6fplj7swdatbv5oa:3:10:1234")
         u1 = uri.ImmutableDirectoryURI(fnuri)
         self.failUnless(u1.is_readonly())
         self.failIf(u1.is_mutable())
@@ -422,11 +436,11 @@ class Dirnode(unittest.TestCase):
         u1_filenode = u1.get_filenode_cap()
         self.failIf(u1_filenode.is_mutable())
         self.failUnless(u1_filenode.is_readonly())
-        self.failUnlessEqual(u1_filenode.to_string(), fncap)
+        self.failUnlessReallyEqual(u1_filenode.to_string(), fncap)
         self.failUnless(str(u1))
 
         u2 = uri.from_string(u1.to_string())
-        self.failUnlessEqual(u1.to_string(), u2.to_string())
+        self.failUnlessReallyEqual(u1.to_string(), u2.to_string())
         self.failUnless(u2.is_readonly())
         self.failIf(u2.is_mutable())
         self.failUnless(IURI.providedBy(u2))
@@ -434,14 +448,14 @@ class Dirnode(unittest.TestCase):
         self.failUnless(IDirnodeURI.providedBy(u2))
 
         u2i = uri.from_string(u1.to_string(), deep_immutable=True)
-        self.failUnlessEqual(u1.to_string(), u2i.to_string())
+        self.failUnlessReallyEqual(u1.to_string(), u2i.to_string())
 
         u3 = u2.get_readonly()
-        self.failUnlessEqual(u3.to_string(), u2.to_string())
+        self.failUnlessReallyEqual(u3.to_string(), u2.to_string())
         self.failUnless(str(u3))
 
         u3i = uri.from_string(u2.to_string(), deep_immutable=True)
-        self.failUnlessEqual(u2.to_string(), u3i.to_string())
+        self.failUnlessReallyEqual(u2.to_string(), u3i.to_string())
 
         u2_verifier = u2.get_verify_cap()
         self.failUnless(isinstance(u2_verifier,
@@ -455,20 +469,20 @@ class Dirnode(unittest.TestCase):
         self.failUnless(IVerifierURI.providedBy(u2_verifier_fileuri))
         u2vfs = u2_verifier_fileuri.to_string()
         # URI:CHK-Verifier:$key:$ueb:$k:$n:$size
-        self.failUnlessEqual(u2vfs, fnuri.get_verify_cap().to_string())
-        self.failUnlessEqual(u2vs[len("URI:DIR2-"):], u2vfs[len("URI:"):])
+        self.failUnlessReallyEqual(u2vfs, fnuri.get_verify_cap().to_string())
+        self.failUnlessReallyEqual(u2vs[len("URI:DIR2-"):], u2vfs[len("URI:"):])
         self.failUnless(str(u2_verifier))
 
     def test_literal(self):
         u0 = uri.LiteralFileURI("data")
         u1 = uri.LiteralDirectoryURI(u0)
         self.failUnless(str(u1))
-        self.failUnlessEqual(u1.to_string(), "URI:DIR2-LIT:mrqxiyi")
+        self.failUnlessReallyEqual(u1.to_string(), "URI:DIR2-LIT:mrqxiyi")
         self.failUnless(u1.is_readonly())
         self.failIf(u1.is_mutable())
         self.failUnless(IURI.providedBy(u1))
         self.failIf(IFileURI.providedBy(u1))
         self.failUnless(IDirnodeURI.providedBy(u1))
-        self.failUnlessEqual(u1.get_verify_cap(), None)
-        self.failUnlessEqual(u1.get_storage_index(), None)
-        self.failUnlessEqual(u1.abbrev_si(), "<LIT>")
+        self.failUnlessReallyEqual(u1.get_verify_cap(), None)
+        self.failUnlessReallyEqual(u1.get_storage_index(), None)
+        self.failUnlessReallyEqual(u1.abbrev_si(), "<LIT>")
